@@ -21,10 +21,11 @@ import pathlib
 import sys
 from flask import Flask
 from optparse import OptionParser
+import logging
 
 SLIDE_DIR = "/mnt/data/shared/"
-SLIDE_CACHE_SIZE = 10
-DEEPZOOM_FORMAT = 'jpeg'
+SLIDE_CACHE_SIZE = 60
+DEEPZOOM_FORMAT = 'png'
 DEEPZOOM_TILE_SIZE = 254
 DEEPZOOM_OVERLAP = 1
 DEEPZOOM_LIMIT_BOUNDS = True
@@ -37,29 +38,27 @@ PLUGINS = []
 if getattr(sys, 'frozen', False):
     template_folder=os.path.join(sys._MEIPASS, 'templates')
     static_folder=os.path.join(sys._MEIPASS, 'static')
+    plugins_folder=os.path.join(sys._MEIPASS, 'plugins')
     os.chdir(sys._MEIPASS)
-else: #if __file__:
-    #template_folder="templates_standalone"
+else:
     folderPath = os.path.dirname(pathlib.Path(__file__))
     template_folder=os.path.join(folderPath, 'templates')
     static_folder=os.path.join(folderPath, 'static')
-    os.chdir(folderPath)
+    plugins_folder=os.path.join(folderPath, 'plugins')
+
+logging.info("template_folder: " + template_folder)
+logging.info("static_folder: " + static_folder)
 
-print ("template_folder",template_folder)
-print ("static_folder",static_folder)
 app = Flask(__name__,template_folder=template_folder,static_folder=static_folder)
 app.config.from_object(__name__)
 app.config.from_envvar('DEEPZOOM_MULTISERVER_SETTINGS', silent=True)
+app.config["PLUGIN_FOLDER"] = plugins_folder
 
-if getattr(sys, 'frozen', False):
-    plugins_folder=os.path.join(sys._MEIPASS, 'plugins')
-elif __file__:
-    plugins_folder="plugins"
-
-for module in glob.glob(plugins_folder + "/*.py"):
+for module in glob.glob(app.config["PLUGIN_FOLDER"] + "/*.py"):
     if "__init__.py" in module:
         continue
     app.config["PLUGINS"].append(os.path.splitext(os.path.basename(module))[0])
+logging.info("Plugin list:",app.config["PLUGINS"])
 
 app.config["isStandalone"] = False
 
